unit uDrawVisitor;

interface

uses
  uPattern                        // for TVisitor
  , uHouse                        // for THouse, TWindow, TDoor
  , graphics                      // for TCanvas
  , windows                       // for TRect
  ;

type

  TDrawVisitor = class(TVisitor)
  private

    FCanvas: TCanvas;

    procedure DrawHouse(Visited: TVisited);
    procedure DrawWindow(Visited: TVisited);
    procedure DrawDoor(Visited: TVisited);
    procedure WriteMaterial(Visited: TVisited; Rect: TRect);
    procedure WriteType(Visited: TVisited; Rect: TRect);

  public

    procedure AllowVisitorToDoSomethingTo(Visited: TVisited); override;
    property Canvas: TCanvas read FCanvas write FCanvas;

  end;

procedure DrawThing(Thing: TVisited; Canvas: TCanvas);

implementation

//******************* DrawThing *************************
// Helper routine to make the visitor easier to use
procedure DrawThing(Thing: TVisited; Canvas: TCanvas);
var
  Visitor: TDrawVisitor;
begin
  Visitor := TDrawVisitor.Create;
  Visitor.Canvas := Canvas;
  Thing.AcceptVisitFrom(Visitor);
end;

{ TDrawVisitor }

procedure TDrawVisitor.AllowVisitorToDoSomethingTo(Visited: TVisited);
begin
  Assert(Assigned(FCanvas), 'TDrawVisitor - I haven''t got a Canvas yet');

  if (Visited is THouse) then
  begin
    Self.DrawHouse(Visited);
    Exit;
  end;

  if (Visited is TWindow) then
  begin
    Self.DrawWindow(Visited);
    Exit;
  end;

  if (Visited is TDoor) then
  begin
    Self.DrawDoor(Visited);
    Exit;
  end;

end;

procedure TDrawVisitor.DrawDoor(Visited: TVisited);
var
  Left, Top, Right, Bottom: integer;
  paneWidth, paneHeight, paneLeft, paneTop, paneRight, paneBottom: integer;
  DoorRect: PRect;
begin
  // Work out the door's coordinates
  Left := TDoor(Visited).House.Left + TDoor(Visited).Left;
  Top := TDoor(Visited).House.Top + TDoor(Visited).Top;
  Right := Left + TDoor(Visited).Width;
  Bottom := Top + TDoor(Visited).Height;
  // Draw the door
  Canvas.Pen.Mode := pmCopy;
  Canvas.Brush.Color := TDoor(Visited).Colour;
  Canvas.Brush.Style := bsSolid;
  Canvas.Rectangle(Left, Top, Right, Bottom);
  // Draw 1 window panes
  paneWidth := TDoor(Visited).Width div 2;
  paneHeight := TDoor(Visited).Height div 3;
  // Work out the pane's coordinates
  paneLeft := Left + ((TDoor(Visited).Width - paneWidth) div 2);
  paneTop := Top + ((TDoor(Visited).Width - paneWidth) div 2);
  paneRight := paneLeft + paneWidth;
  paneBottom := paneTop + paneHeight;
  // Draw the window pane
  Canvas.Brush.Color := clAqua;         // panes of glass are always Aqua
  Canvas.Brush.Style := bsSolid;
  Canvas.Rectangle(paneLeft, paneTop, paneRight, paneBottom);
  // Write the window material type in the middle and the THoleStopper type
  // at the top
  New(DoorRect);
  try
    DoorRect^.Left := Left;
    DoorRect^.Top := Top;
    DoorRect^.Right := Right;
    DoorRect^.Bottom := Bottom;
    WriteMaterial(Visited, DoorRect^);
    WriteType(Visited, DoorRect^);
  finally
    Dispose(DoorRect);
  end;
end;

procedure TDrawVisitor.DrawHouse(Visited: TVisited);
begin
  // Draw the window pane
  Canvas.Brush.Color := clRed;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Mode := pmCopy;
  Canvas.Rectangle(THouse(Visited).Left, THouse(Visited).Top,
                   THouse(Visited).Left + THouse(Visited).Width,
                   THouse(Visited).Top + THouse(Visited).Height);
end;

procedure TDrawVisitor.DrawWindow(Visited: TVisited);
var
  Left, Top, Right, Bottom: integer;
  paneWidth, paneHeight, paneLeft, paneTop, paneRight, paneBottom: integer;
  nPane: integer;
  WindowRect: PRect;
begin
  // Work out the window's coordinates
  Left := TWindow(Visited).House.Left + TWindow(Visited).Left;
  Top := TWindow(Visited).House.Top + TWindow(Visited).Top;
  Right := Left + TWindow(Visited).Width;
  Bottom := Top + TWindow(Visited).Height;
  // Draw the window frame
  Canvas.Pen.Mode := pmCopy;
  Canvas.Brush.Color := TWindow(Visited).Colour;
  Canvas.Brush.Style := bsSolid;
  Canvas.Rectangle(Left, Top, Right, Bottom);
  // Draw 4 window panes
  paneWidth := (TWindow(Visited).Width - 30) div 2;
  paneHeight := (TWindow(Visited).Height - 30) div 2;
  for nPane := 0 to 3 do
  begin
    // Work out the pane's coordinates
    paneLeft := Left + 10 + ((paneWidth + 10) * (nPane mod 2));
    paneTop := Top + 10 + ((paneHeight + 10) * (nPane div 2));
    paneRight := paneLeft + paneWidth;
    paneBottom := paneTop + paneHeight;
    // Draw the window pane
    Canvas.Brush.Color := clAqua;         // panes of glass are always Aqua
    Canvas.Brush.Style := bsSolid;
    Canvas.Rectangle(paneLeft, paneTop, paneRight, paneBottom);
  end;
  // Write the window material type in the middle
  New(WindowRect);
  try
    WindowRect^.Left := Left;
    WindowRect^.Top := Top;
    WindowRect^.Right := Right;
    WindowRect^.Bottom := Bottom;
    WriteMaterial(Visited, WindowRect^);
    WriteType(Visited, WindowRect^);
  finally
    Dispose(WindowRect);
  end;
end;

procedure TDrawVisitor.WriteMaterial(Visited: TVisited; Rect: TRect);
var
  txtWidth, txtHeight, x, y: integer;
  Text: string;
begin
  // Get the string to display
  case THoleStopper(Visited).Material of
    tmWood: Text := 'Wood';
    tmPlastic: Text := 'Plastic';
  end;
  // Find out where to put it
  txtWidth := Canvas.TextWidth(Text);
  txtHeight := Canvas.TextHeight(Text);
  x := Rect.Left + ((Rect.Right - Rect.Left) div 2) - (txtWidth div 2);
  y := Rect.Top + ((Rect.Bottom - Rect.Top) div 2) - (txtHeight div 2);
  // Write the text
  Canvas.Pen.Mode := pmNot;                     // Draw in the inverse colour
  Canvas.TextOut(x, y, Text);
end;

procedure TDrawVisitor.WriteType(Visited: TVisited; Rect: TRect);
var
  txtWidth, x, y: integer;
  Text: string;
begin
  // Get the string to display
  Text := THoleStopper(Visited).StopperType;
  // Find out where to put it
  txtWidth := Canvas.TextWidth(Text);
  x := Rect.Left + ((Rect.Right - Rect.Left) div 2) - (txtWidth div 2);
  y := Rect.Top + 5;
  // Write the text
  Canvas.Pen.Mode := pmNot;                     // Draw in the inverse colour
  Canvas.TextOut(x, y, Text);
end;

end.
